package api

import (
	"fmt"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func RegMeshCrud(router *GinRouter) {

	//创建Mesh
	CreateCRUD(router, "/mesh", &CRUDOption{
		Collection: repo.CollectionMesh,
		NewModel: func(c *gin.Context) interface{} {
			body := &struct {
				Category  string
				Name      string
				Thumbnail *model.OssType
				File      *model.OssType
			}{}
			c.ShouldBindJSON(&body)

			out := &model.Mesh{
				Name:       body.Name,
				Category:   body.Category,
				CreateTime: time.Now(),
				Geometries: []string{},
				File:       body.File,
				Thumbnail:  body.Thumbnail,
			}

			return out
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.Mesh{}
		},
		//DetailProject: []string{"osgjs", "name", "file", "materials"},
		SearchProject: []string{"name", "createTime", "geometries", "thumbnail", "osgjs", "category", "osgjsbin", "file", "geometries"},
	})

	router.GET("/mesh/detail/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		id := c.Param("id")
		if len(id) < 1 {
			return nil, NewError("id不能为空!")
		}
		out := &model.Mesh{}

		uid, _ := primitive.ObjectIDFromHex(id)

		ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: repo.CollectionMesh,
			Query:       repo.Map{"_id": uid},
		}, out)

		if err != nil {
			return nil, err
		}
		if !ok {
			return nil, NewError("没有查询到数据")
		}
		return out, nil
	})

	//转换模型
	// router.POST("/mesh/conv", ServiceConvModel)

	//鞋型编辑
	CreateCRUD(router, "/shoe", &CRUDOption{
		Collection: repo.CollectionShoes,

		NewModel: func(c *gin.Context) interface{} {
			body := &struct {
				Sex       uint8
				Type      string
				Season    uint8
				Name      string
				Thumbnail *model.OssType
			}{}
			c.ShouldBindJSON(&body)

			nopublic := false
			Platform := true

			return &model.ShoeMesh{
				Name:       body.Name,
				Sex:        body.Sex,
				Type:       body.Type,
				Season:     body.Season,
				IsPublic:   &nopublic,
				Thumbnail:  body.Thumbnail,
				Platform:   &Platform,
				CreateTime: time.Now(),
			}
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.ShoeMesh{}
		},
		SearchFilter: func(c *gin.Context, apictx *ApiSession, query map[string]interface{}) map[string]interface{} {
			return map[string]interface{}{"platform": true}
		},
		SearchProject: []string{"name", "thumbnail", "isPublic", "createTime", "sex", "type", "season", "meshId"},
	})

	//鞋楦
	CreateCRUD(router, "/last", &CRUDOption{
		Collection: repo.CollectionLasts,
		NewModel: func(c *gin.Context) interface{} {
			body := &struct {
				Sex       uint8
				Type      string
				Season    uint8
				Name      string
				Thumbnail *model.OssType
			}{}
			c.ShouldBindJSON(&body)

			nopublic := false
			Platform := true

			return &model.LastMesh{
				Name:       body.Name,
				Sex:        body.Sex,
				Type:       body.Type,
				Season:     body.Season,
				IsPublic:   &nopublic,
				Thumbnail:  body.Thumbnail,
				Platform:   &Platform,
				CreateTime: time.Now(),
			}
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.LastMesh{}
		},
		SearchFilter: func(c *gin.Context, apictx *ApiSession, query map[string]interface{}) map[string]interface{} {
			return map[string]interface{}{"platform": true}
		},
		SearchProject: []string{"name", "thumbnail", "isPublic", "createTime", "sex", "type", "season", "meshId"},
	})

	//鞋跟
	CreateCRUD(router, "/heel", &CRUDOption{
		Collection: repo.CollectionHeels,
		NewModel: func(c *gin.Context) interface{} {
			body := &struct {
				Sex       uint8
				Type      string
				Season    uint8
				Name      string
				Thumbnail *model.OssType
			}{}
			c.ShouldBindJSON(&body)

			nopublic := false
			Platform := true

			return &model.HeelMesh{
				Name:       body.Name,
				Sex:        body.Sex,
				Type:       body.Type,
				Season:     body.Season,
				IsPublic:   &nopublic,
				Thumbnail:  body.Thumbnail,
				Platform:   &Platform,
				CreateTime: time.Now(),
			}
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.HeelMesh{}
		},
		SearchFilter: func(c *gin.Context, apictx *ApiSession, query map[string]interface{}) map[string]interface{} {
			return map[string]interface{}{"platform": true}
		},
		SearchProject: []string{"name", "thumbnail", "isPublic", "createTime", "sex", "type", "season", "meshId"},
	})

	//大底
	CreateCRUD(router, "/sole", &CRUDOption{
		Collection: repo.CollectionSoles,
		NewModel: func(c *gin.Context) interface{} {
			body := &struct {
				Sex       uint8
				Type      string
				Season    uint8
				Name      string
				Thumbnail *model.OssType
			}{}
			c.ShouldBindJSON(&body)

			nopublic := false
			Platform := true

			return &model.SoleMesh{
				Name:       body.Name,
				Sex:        body.Sex,
				Type:       body.Type,
				Season:     body.Season,
				IsPublic:   &nopublic,
				Thumbnail:  body.Thumbnail,
				Platform:   &Platform,
				CreateTime: time.Now(),
			}
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.SoleMesh{}
		},
		SearchFilter: func(c *gin.Context, apictx *ApiSession, query map[string]interface{}) map[string]interface{} {
			return map[string]interface{}{"platform": true}
		},
		SearchProject: []string{"name", "thumbnail", "isPublic", "createTime", "sex", "type", "season", "meshId"},
	})

	//配饰
	CreateCRUD(router, "/decorate", &CRUDOption{
		Collection: repo.CollectionDecorates,
		NewModel: func(c *gin.Context) interface{} {
			body := &struct {
				Sex       uint8
				Type      string
				Season    uint8
				Name      string
				Thumbnail *model.OssType
			}{}
			c.ShouldBindJSON(&body)

			nopublic := false
			Platform := true

			return &model.DecorateMesh{
				Name:       body.Name,
				Sex:        body.Sex,
				Type:       body.Type,
				Season:     body.Season,
				IsPublic:   &nopublic,
				Thumbnail:  body.Thumbnail,
				Platform:   &Platform,
				CreateTime: time.Now(),
			}
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.DecorateMesh{}
		},
		SearchFilter: func(c *gin.Context, apictx *ApiSession, query map[string]interface{}) map[string]interface{} {
			return map[string]interface{}{"platform": true}
		},
		SearchProject: []string{"name", "thumbnail", "isPublic", "createTime", "sex", "type", "season", "meshId"},
	})

	router.POSTJWT("/lib/update", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		body := &struct {
			Id         string `json:"_id"`
			Type       string
			Name       string
			CusNum     string
			Thumbnail  *model.OssType
			State      int32
			Categories []string
			Scope      string
		}{}
		c.ShouldBindJSON(&body)

		collections := map[string]string{"imgmat": repo.CollectionImageMat, "fabric": repo.CollectionFabric, "sticker": repo.CollectionStickers, "design": repo.CollectionDesigns, "last": repo.CollectionLasts, "heel": repo.CollectionHeels, "sole": repo.CollectionSoles, "decorate": repo.CollectionDecorates}
		if len(collections[body.Type]) < 1 {
			return nil, NewError("库类型不支持!")
		}

		if len(body.Id) < 1 {
			return nil, NewError("Id不能为空!")
		}

		collect := collections[body.Type]
		if body.Scope == "team" || body.Scope == "company" {
			collect = fmt.Sprintf("%s_%s", body.Type, body.Scope)
		}

		fmt.Println("update=>", body)
		data := bson.M{"state": body.State}
		if body.Thumbnail != nil {
			data["thumbnail"] = body.Thumbnail
		}
		if body.Categories != nil && len(body.Categories) > 0 {
			data["categories"] = body.Categories
		}

		if len(body.Name) > 0 {
			data["name"] = body.Name
		}

		if len(body.CusNum) > 0 {
			data["cusNum"] = body.CusNum
		}

		if body.Scope == "company" && body.State == model.LibState_Publish2Qiye && len(body.Name) < 1 { //更新发布状态
			data["createTime"] = time.Now()
		}

		return repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), collect, body.Id, bson.M{"$set": data})
	})

	router.GET("/lib/mesh/osg", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

		id := c.Query("id")
		if len(id) < 1 {
			return nil, NewError("id不能为空!")
		}
		meshType := c.Query("type")
		if meshType == "soles" {
			meshType = "sole"
		}
		scope := c.Query("scope")
		// if len(scope) < 1 {
		// 	return nil, NewError("预览模型的scope不能为空!")
		// }

		collections := map[string]string{"shoes": repo.CollectionShoes, "last": repo.CollectionLasts, "heel": repo.CollectionHeels, "sole": repo.CollectionSoles, "decorate": repo.CollectionDecorates}
		CollectName := collections[meshType]
		if len(CollectName) < 1 {
			return nil, NewError("库类型不支持!")
		}

		if scope == "team" || scope == "company" {
			if meshType != "shoes" {
				CollectName = fmt.Sprintf("%s_%s", meshType, scope)
			}
		}

		if meshType == "shoes" {
			designCollection := repo.CollectionDesigns
			if scope == "team" || scope == "company" {
				designCollection = fmt.Sprintf("design_%s", scope)
			}
			ok, shoeRet := repo.RepoSeachDocMap(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
				CollectName: designCollection,
				Query:       repo.Map{"_id": id},
				Project:     []string{"shoeId"},
			})
			if !ok {
				return nil, NewError("模型查询失败!")
			}
			id = shoeRet["shoeId"].(primitive.ObjectID).Hex()
		}

		ok, meshRet := repo.RepoSeachDocMap(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: CollectName,
			Query:       repo.Map{"_id": id},
			Project:     []string{"meshId"},
		})
		if !ok {
			return nil, NewError("模型查询失败!")
		}
		meshId, _ := meshRet["meshId"].(primitive.ObjectID)

		ok, meshDetail := repo.RepoSeachDocMap(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: repo.CollectionMesh,
			Query:       repo.Map{"_id": meshId},
			Project:     []string{"osgjs", "name", "file", "shadow"},
		})

		if !ok {
			return nil, NewError("模型查询失败!")
		}

		return meshDetail, nil
	})

	// router.POSTJWT("/mesh/shadow/create/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	// 	designId := c.Param("id")
	// 	if len(designId) < 1 {
	// 		return nil, NewError("id不能为空!")
	// 	}

	// 	ok, shoeRet := repo.RepoSeachDocMap(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
	// 		CollectName: repo.CollectionDesigns,
	// 		Query:       repo.Map{"_id": designId},
	// 		Project:     []string{"shoeId"},
	// 	})

	// 	if !ok {
	// 		return nil, NewError("模型查询失败!")
	// 	}
	// 	id := shoeRet["shoeId"].(primitive.ObjectID).Hex()

	// 	ok, meshShoeRet := repo.RepoSeachDocMap(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
	// 		CollectName: repo.CollectionShoes,
	// 		Query:       repo.Map{"_id": id},
	// 		Project:     []string{"meshId"},
	// 	})
	// 	if !ok {
	// 		return nil, NewError("模型查询失败!")
	// 	}
	// 	meshId, _ := meshShoeRet["meshId"].(primitive.ObjectID)

	// 	meshObj := &model.Mesh{}

	// 	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
	// 		CollectName: repo.CollectionMesh,
	// 		Query:       repo.Map{"_id": meshId.Hex()},
	// 		Project:     []string{"file"},
	// 	}, meshObj)

	// 	if !ok || err != nil {
	// 		return nil, NewError("模型查询失败!")
	// 	}

	// 	return true, bus.NatsCenter.ShadowProcessReq(meshId.Hex(), meshObj.File.Url, apictx.User.ID)
	// })

}
