package api

import (
	"fmt"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

const (
	PLATFORM_USER_ID = "61c2d599562300001a004978"
	DADONG_USER_ID   = "637cfe24f6b64a24e5a52da7"
)

func RegLib(router *GinRouter, collection string, prefix string, project []string) {

	platform := fmt.Sprintf("/platform/public/%s/list", prefix)
	router.GET(platform, func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		return QiyeLibList(PLATFORM_USER_ID, prefix, c, apictx, project)
	})

	dadong := fmt.Sprintf("/dadong/public/%s/list", prefix)
	router.GET(dadong, func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		return QiyeLibList(DADONG_USER_ID, prefix, c, apictx, project)
	})

	qiye := fmt.Sprintf("/qiye/public/%s/list", prefix)
	router.GET(qiye, func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		_, _, query := UtilQueryPageSize(c)
		id := query["id"].(string)
		query["id"] = nil
		return QiyeLibList(id, prefix, c, apictx, project)
	})

	//qiye的数据，走企业的库
	uri := fmt.Sprintf("/qiye/%s/list", prefix)
	router.GETJWT(uri, func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

		page, size, query := UtilQueryPageSize(c)

		query["companyId"] = apictx.User.Parent

		ParseCategories(query, apictx)

		teamColl := fmt.Sprintf("%s_company", prefix)

		out, err := repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
			CollectName: teamColl,
			Page:        page,
			Size:        size,
			Query:       query,
			Project:     project,
			Sort:        bson.M{"createTime": -1},
		})
		if err != nil {
			return nil, err
		}

		for _, v := range out.List {
			if v["userId"] != nil {
				userId := v["userId"].(primitive.ObjectID).Hex()
				u, _ := repo.RedisGetUserById(apictx.CreateRepoCtx(), userId)
				if u != nil {
					v["user"] = u //{avatar:"", name:""}
				}
			}
		}

		return out, nil
	})

	//部门的数据
	teamUri := fmt.Sprintf("/team/%s/list", prefix)
	router.GETJWT(teamUri, func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		page, size, query := UtilQueryPageSize(c)

		//获取teamId
		//query["userId"] = bson.M{"$in": repo.TeamGetUserTeamers(apictx.CreateRepoCtx(), apictx.User.ID, true)}
		// if query["state"] == nil {
		// 	query["state"] = bson.M{"$gt": model.LibState_Created}
		// }
		teamId := repo.TeamGetUserTeamId(apictx.CreateRepoCtx(), apictx.User.ID)
		if len(teamId) < 1 {
			return nil, NewError("当前用户没有加入任何部门!")
		}
		query["teamId"] = teamId

		ParseCategories(query, apictx)

		teamColl := fmt.Sprintf("%s_team", prefix)

		out, err := repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
			CollectName: teamColl,
			Page:        page,
			Size:        size,
			Query:       query,
			Project:     project,
			Sort:        bson.M{"createTime": -1},
		})
		if err != nil {
			return nil, err
		}

		for _, v := range out.List {
			if v["userId"] != nil {
				userId := v["userId"].(primitive.ObjectID).Hex()
				u, _ := repo.RedisGetUserById(apictx.CreateRepoCtx(), userId)
				if u != nil {
					v["user"] = u
				}
			}
		}

		return out, nil
	})

	user := fmt.Sprintf("/user/%s/list", prefix)
	router.GETJWT(user, func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		page, size, query := UtilQueryPageSize(c)

		uid, _ := primitive.ObjectIDFromHex(apictx.User.ID)
		query["userId"] = uid

		ParseCategories(query, apictx)

		return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
			CollectName: collection,
			Page:        page,
			Size:        size,
			Query:       query,
			Project:     project,
			Sort:        bson.M{"createTime": -1},
		})
	})
}

func ParseCategories(query map[string]interface{}, apictx *ApiSession) {

	//keyword
	if query["keyword"] != nil {
		keyword := query["keyword"].(string)
		if len(keyword) > 0 {
			query["name"] = bson.M{"$regex": keyword, "$options": "$i"}
		}
		query["keyword"] = nil
	}

	if query["categories"] == nil {
		return
	}

	categories := query["categories"].([]interface{})
	//查询所有子内容
	if len(categories) == 0 {
		query["categories"] = nil
		return
	}

	filters := []bson.M{}
	for _, c := range categories {
		extends := repo.CategoryExtend([]string{c.(string)}, apictx.CreateRepoCtx())
		if len(extends) > 0 {
			filter := bson.M{"categories": bson.M{"$elemMatch": bson.M{"$in": extends}}}
			filters = append(filters, filter)
		}
	}
	query["categories"] = nil
	if len(filters) > 0 {
		query["$and"] = filters
	}
}

func QiyeLibList(id string, prefix string, c *gin.Context, apictx *ApiSession, project []string) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)

	query["companyId"] = id
	query["state"] = model.LibState_Publish2Qiye

	ParseCategories(query, apictx)

	teamColl := fmt.Sprintf("%s_company", prefix)

	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: teamColl,
		Page:        page,
		Size:        size,
		Query:       query,
		Project:     project,
		Sort:        bson.M{"createTime": -1},
	})
}

func CreateUserUploadRouter(router *GinRouter) {

	//用户款式列表
	router.POSTJWT("/resource/delete", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		body := &struct {
			Scope string
			Type  string
			Id    string
		}{}
		c.ShouldBindJSON(body)
		if len(body.Id) < 1 {
			return nil, NewError("id不能为空!")
		}
		collectionName := repo.CollectionDesigns
		switch body.Type {
		case "design":
			collectionName = repo.CollectionDesigns
		case "heel":
			collectionName = repo.CollectionHeels
		case "sole":
			collectionName = repo.CollectionSoles
		case "last":
			collectionName = repo.CollectionLasts
		case "decorate":
			collectionName = repo.CollectionDecorates

		case "imgmat":
			collectionName = repo.CollectionImageMat
		case "fabric":
			collectionName = repo.CollectionFabric

		case "sticker":
			collectionName = repo.CollectionStickers
		}

		if body.Scope == "team" || body.Scope == "company" {
			collectionName = fmt.Sprintf("%s_%s", body.Type, body.Scope)
		}

		ret, err := repo.RepoDeleteDoc(apictx.CreateRepoCtx(), collectionName, body.Id)
		if err != nil || ret.DeletedCount != 1 {
			return nil, NewError("删除失败!")
		}

		return ret, err
	})

	RegLib(router, repo.CollectionDesigns, "design", []string{"name", "cusNum", "thumbnail", "createTime", "state", "categories", "meshState", "userId"})

	//上传款式
	// router.POSTJWT("/upload/shoe", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	// 	body := &struct {
	// 		Name       string
	// 		File       *model.OssType
	// 		Thumbnail  *model.OssType
	// 		Categories []string
	// 	}{}
	// 	c.ShouldBindJSON(body)
	// 	if body.File == nil || body.File.Size < 1 || len(body.File.Url) < 1 {
	// 		return nil, NewError("模型文件不能为空")
	// 	}
	// 	if len(body.Categories) < 1 {
	// 		return nil, NewError("分类不能为空!")
	// 	}

	// 	nopublic := false
	// 	Platform := true

	// 	shoe := &model.ShoeMesh{
	// 		Name:       body.Name,
	// 		IsPublic:   &nopublic,
	// 		Thumbnail:  body.Thumbnail,
	// 		Platform:   &Platform,
	// 		CreateTime: time.Now(),
	// 	}
	// 	shoe.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)

	// 	//生成meshId
	// 	mesh := &model.Mesh{MeshState: Int32Value(0), Name: body.Name, Thumbnail: body.Thumbnail, CreateTime: time.Now(), Category: "shoe", File: body.File}
	// 	meshId, errr := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionMesh, mesh)
	// 	if errr != nil {
	// 		return nil, errr
	// 	}

	// 	shoe.MeshId, _ = primitive.ObjectIDFromHex(meshId)
	// 	shoeid, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionShoes, shoe)
	// 	if err != nil {
	// 		return nil, err
	// 	}

	// 	//生成款式设计
	// 	design, err := createDefaultDesign(apictx, shoeid)
	// 	if err != nil {
	// 		return nil, err
	// 	}
	// 	design.UserId = shoe.UserId
	// 	design.Categories = body.Categories
	// 	design.State = Int32Value(model.LibState_Created)
	// 	design.MeshState = Int32Value(model.MeshState_SetFile)

	// 	designId, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionDesigns, design)
	// 	if err != nil {
	// 		return nil, err
	// 	}

	// 	//提交模型转换任务
	// 	errRet := bus.NatsCenter.MeshAssetProcessReq(repo.CollectionDesigns, designId, meshId, body.File.Url, apictx.User.ID, true)
	// 	if errRet != nil { //不成功!
	// 		//设置处理失败状态
	// 		repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), repo.CollectionDesigns, designId, bson.M{"$set": bson.M{"meshState": model.MeshState_SetOsgjsFailded}})

	// 		return nil, errRet
	// 	}

	// 	//task.RunMeshPostProcessTask(meshId, repo.CollectionMesh, designId, repo.CollectionDesigns)

	// 	// taskId, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionTask, taskConfig)
	// 	// if err != nil {
	// 	// 	return nil, err
	// 	// }

	// 	// collection := repo.CollectionTask

	// 	// req := &model.WorkerRequest{WorkerType: "sku3dRender", MainId: taskConfig.DesignId.Hex(), TaskCollection: collection, UserId: apictx.User.ID, Name: "图片渲染", TaskId: taskId}

	// 	//  ReqestTaskWorker(apictx.CreateRepoCtx(), req, collection)

	// 	return designId, nil
	// })

	meshProject := []string{"name", "thumbnail", "createTime", "meshId", "state", "cusNum", "categories", "userId"}
	RegLib(router, repo.CollectionLasts, "last", meshProject)

	//用户款式列表
	// router.POSTJWT("/upload/last", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	// 	body := &struct {
	// 		Name       string
	// 		File       *model.OssType
	// 		Thumbnail  *model.OssType
	// 		Categories []string
	// 	}{}
	// 	c.ShouldBindJSON(body)
	// 	if body.File == nil || body.File.Size < 1 || len(body.File.Url) < 1 {
	// 		return nil, NewError("模型文件不能为空")
	// 	}
	// 	nopublic := false
	// 	Platform := true

	// 	shoe := &model.LastMesh{
	// 		Name:       body.Name,
	// 		IsPublic:   &nopublic,
	// 		Thumbnail:  body.Thumbnail,
	// 		Platform:   &Platform,
	// 		CreateTime: time.Now(),
	// 		Categories: body.Categories,
	// 		State:      model.LibState_Created,
	// 	}
	// 	shoe.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)
	// 	shoe.MeshState = Int32Value(model.MeshState_SetFile)

	// 	//生成meshId
	// 	mesh := &model.Mesh{Name: body.Name, Thumbnail: body.Thumbnail, CreateTime: time.Now(), Category: "shoe", File: body.File}
	// 	meshId, errr := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionMesh, mesh)
	// 	if errr != nil {
	// 		return nil, errr
	// 	}
	// 	shoe.MeshId, _ = primitive.ObjectIDFromHex(meshId)

	// 	lastId, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionLasts, shoe)
	// 	if err != nil {
	// 		return nil, err
	// 	}

	// 	//模型转换
	// 	errRet := bus.NatsCenter.MeshAssetProcessReq(repo.CollectionLasts, lastId, meshId, body.File.Url, apictx.User.ID, false)
	// 	if errRet != nil { //不成功!
	// 		//设置处理失败状态
	// 		repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), repo.CollectionLasts, lastId, bson.M{"$set": bson.M{"meshState": model.MeshState_SetOsgjsFailded}})

	// 		return nil, errRet
	// 	}

	// 	return lastId, err
	// })

	RegLib(router, repo.CollectionHeels, "heel", meshProject)
	// router.POSTJWT("/upload/heel", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	// 	body := &struct {
	// 		Name       string
	// 		File       *model.OssType
	// 		Thumbnail  *model.OssType
	// 		Categories []string
	// 	}{}
	// 	c.ShouldBindJSON(body)
	// 	if body.File == nil || body.File.Size < 1 || len(body.File.Url) < 1 {
	// 		return nil, NewError("模型文件不能为空")
	// 	}
	// 	nopublic := false
	// 	Platform := true

	// 	shoe := &model.HeelMesh{
	// 		Name:       body.Name,
	// 		IsPublic:   &nopublic,
	// 		Thumbnail:  body.Thumbnail,
	// 		Platform:   &Platform,
	// 		CreateTime: time.Now(),
	// 		Categories: body.Categories,
	// 		State:      model.LibState_Created,
	// 	}
	// 	shoe.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)
	// 	shoe.MeshState = Int32Value(model.MeshState_SetFile)

	// 	//生成meshId
	// 	mesh := &model.Mesh{Name: body.Name, Thumbnail: body.Thumbnail, CreateTime: time.Now(), Category: "shoe", File: body.File}
	// 	meshId, errr := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionMesh, mesh)
	// 	if errr != nil {
	// 		return nil, errr
	// 	}
	// 	shoe.MeshId, _ = primitive.ObjectIDFromHex(meshId)

	// 	heelId, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionHeels, shoe)
	// 	if err != nil {
	// 		return nil, err
	// 	}

	// 	//模型转换
	// 	errRet := bus.NatsCenter.MeshAssetProcessReq(repo.CollectionHeels, heelId, meshId, body.File.Url, apictx.User.ID, false)
	// 	if errRet != nil { //不成功!
	// 		//设置处理失败状态
	// 		repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), repo.CollectionHeels, heelId, bson.M{"$set": bson.M{"meshState": model.MeshState_SetOsgjsFailded}})

	// 		return nil, errRet
	// 	}

	// 	return heelId, err
	// })

	RegLib(router, repo.CollectionSoles, "sole", meshProject)

	// router.POSTJWT("/upload/sole", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	// 	body := &struct {
	// 		Name       string
	// 		File       *model.OssType
	// 		Thumbnail  *model.OssType
	// 		Categories []string
	// 	}{}
	// 	c.ShouldBindJSON(body)
	// 	if body.File == nil || body.File.Size < 1 || len(body.File.Url) < 1 {
	// 		return nil, NewError("模型文件不能为空")
	// 	}
	// 	nopublic := false
	// 	Platform := true

	// 	shoe := &model.SoleMesh{
	// 		Name:       body.Name,
	// 		IsPublic:   &nopublic,
	// 		Thumbnail:  body.Thumbnail,
	// 		Platform:   &Platform,
	// 		CreateTime: time.Now(),
	// 		Categories: body.Categories,
	// 		State:      model.LibState_Created,
	// 	}
	// 	shoe.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)
	// 	shoe.MeshState = Int32Value(model.MeshState_SetFile)

	// 	//生成meshId
	// 	mesh := &model.Mesh{Name: body.Name, Thumbnail: body.Thumbnail, CreateTime: time.Now(), Category: "shoe", File: body.File}
	// 	meshId, errr := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionMesh, mesh)
	// 	if errr != nil {
	// 		return nil, errr
	// 	}
	// 	shoe.MeshId, _ = primitive.ObjectIDFromHex(meshId)

	// 	soleId, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionSoles, shoe)
	// 	if err != nil {
	// 		return nil, err
	// 	}

	// 	errRet := bus.NatsCenter.MeshAssetProcessReq(repo.CollectionSoles, soleId, meshId, body.File.Url, apictx.User.ID, false)
	// 	if errRet != nil { //不成功!
	// 		//设置处理失败状态
	// 		repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), repo.CollectionSoles, soleId, bson.M{"$set": bson.M{"meshState": model.MeshState_SetOsgjsFailded}})

	// 		return nil, errRet
	// 	}

	// 	return soleId, err
	// })

	RegLib(router, repo.CollectionDecorates, "decorate", meshProject)
	// router.POSTJWT("/upload/decorate", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	// 	body := &struct {
	// 		Name       string
	// 		File       *model.OssType
	// 		Thumbnail  *model.OssType
	// 		Categories []string
	// 	}{}
	// 	c.ShouldBindJSON(body)
	// 	if body.File == nil || body.File.Size < 1 || len(body.File.Url) < 1 {
	// 		return nil, NewError("模型文件不能为空")
	// 	}
	// 	nopublic := false
	// 	Platform := true

	// 	shoe := &model.DecorateMesh{
	// 		Name:       body.Name,
	// 		IsPublic:   &nopublic,
	// 		Categories: body.Categories,
	// 		Thumbnail:  body.Thumbnail,
	// 		Platform:   &Platform,
	// 		CreateTime: time.Now(),
	// 		State:      model.LibState_Created,
	// 	}
	// 	shoe.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)
	// 	shoe.MeshState = Int32Value(model.MeshState_SetFile)

	// 	//生成meshId
	// 	mesh := &model.Mesh{Name: body.Name, Thumbnail: body.Thumbnail, CreateTime: time.Now(), Category: "shoe", File: body.File}
	// 	meshId, errr := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionMesh, mesh)
	// 	if errr != nil {
	// 		return nil, errr
	// 	}
	// 	shoe.MeshId, _ = primitive.ObjectIDFromHex(meshId)

	// 	decorateId, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionDecorates, shoe)
	// 	if err != nil {
	// 		return nil, err
	// 	}

	// 	errRet := bus.NatsCenter.MeshAssetProcessReq(repo.CollectionDecorates, decorateId, meshId, body.File.Url, apictx.User.ID, false)
	// 	if errRet != nil { //不成功!
	// 		//设置处理失败状态
	// 		repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), repo.CollectionDecorates, decorateId, bson.M{"$set": bson.M{"meshState": model.MeshState_SetOsgjsFailded}})
	// 		return nil, errRet
	// 	}

	// 	return decorateId, err
	// })

	//2d面料
	RegLib(router, repo.CollectionImageMat, "imgmat", []string{"name", "cusNum", "colorCards", "image", "thumbnail", "userId", "createTime", "isPublic", "categories", "state"})

	RegLib(router, repo.CollectionFabric, "fabric", []string{"name", "cusNum", "colorCards", "thumbnail", "userId", "createTime", "isPublic", "categories", "state"})

	//图片帖子
	RegLib(router, repo.CollectionStickers, "sticker", []string{"name", "cusNum", "image", "createTime", "categories", "state"})
	router.POSTJWT("/upload/sticker", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		body := &struct {
			Name       string
			Image      *model.OssType
			Categories []string
		}{}
		c.ShouldBindJSON(body)
		if body.Image == nil || body.Image.Size < 1 || len(body.Image.Url) < 1 {
			return nil, NewError("图片文件不能为空")
		}

		img := &model.StickerImage{
			Name:       body.Name,
			Image:      body.Image,
			CreateTime: time.Now(),
			Categories: body.Categories,
			State:      model.LibState_Created,
		}
		img.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)

		return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionStickers, img)
	})
}
