package model

import (
	"math"
	"time"

	"sku3dweb/comm"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

const (
	MatType_2D     = 3
	MatType_Color  = 0
	MatType_Effect = 2
	MatType_Fabric = 1
)

// 2D 面料
type Mat2d struct {
	Image       *OssType     `bson:"image,omitempty" json:"image"`
	Name        string       `bson:"name,omitempty" json:"name"`
	CusNum      string       `bson:"cusNum,omitempty" json:"cusNum"`
	Uvtransform *Uvtransform `bson:"uv,omitempty" json:"uv"`
}

type ImageMat struct {
	Id        primitive.ObjectID `bson:"_id,omitempty" json:"_id,omitempty"`
	UserId    primitive.ObjectID `bson:"userId,omitempty" json:"userId,omitempty"`
	TeamId    string             `bson:"teamId,omitempty" json:"teamId,omitempty"`       //所属团队Id
	CompanyId string             `bson:"companyId,omitempty" json:"companyId,omitempty"` //所属团队Id

	Image      *OssType  `bson:"image,omitempty" json:"image,omitempty"`
	Thumbnail  *OssType  `bson:"thumbnail,omitempty" json:"thumbnail,omitempty"`
	Name       string    `bson:"name,omitempty" json:"name,omitempty"`
	CusNum     string    `bson:"cusNum,omitempty" json:"cusNum,omitempty"`
	CreateTime time.Time `bson:"createTime,omitempty" json:"createTime,omitempty"`

	IsPublic *bool `bson:"isPublic,omitempty" json:"isPublic,omitempty"`
	Platform *bool `bson:"platform,omitempty" json:"platform,omitempty"` //是否属于平台

	ColorCards []*Mat2d `bson:"colorCards,omitempty" json:"colorCards,omitempty"` //2D面料
	Categories []string `bson:"categories,omitempty" json:"categories,omitempty"` //目录列表
	State      int32    `bson:"state,omitempty" json:"state,omitempty"`
}

const MIN = 0.0001

// MIN 为用户自定义的比较精度
func IsEqualFloat32(f1, f2 float32) bool {
	return math.Dim(float64(f1), float64(f2)) < MIN
}
func IsEqualFloat64(f1, f2 float64) bool {
	return math.Dim(f1, f2) < MIN
}

func IsEqualboolp(b1p, b2p *bool) bool {
	b1 := false
	b2 := false
	if b1p != nil {
		b1 = *b1p
	}
	if b2p != nil {
		b2 = *b2p
	}
	return b1 == b2
}

func IsEqualFloat64p(f1p, f2p *float64) bool {
	f1 := 0.0
	f2 := 0.0
	if f1p != nil {
		f1 = *f1p
	}
	if f2p != nil {
		f2 = *f2p
	}
	return math.Dim(f1, f2) < MIN
}

type Fabric struct {
	Id        primitive.ObjectID `bson:"_id,omitempty" json:"_id,omitempty"`
	UserId    primitive.ObjectID `bson:"userId,omitempty" json:"userId,omitempty"`
	TeamId    string             `bson:"teamId,omitempty" json:"teamId,omitempty"`       //所属团队Id
	CompanyId string             `bson:"companyId,omitempty" json:"companyId,omitempty"` //所属团队Id

	Name       string    `bson:"name,omitempty" json:"name,omitempty"`
	CusNum     string    `bson:"cusNum,omitempty" json:"cusNum,omitempty"`
	Thumbnail  *OssType  `bson:"thumbnail,omitempty" json:"thumbnail,omitempty"`
	CreateTime time.Time `bson:"createTime,omitempty" json:"createTime,omitempty"`

	Price      float32              `bson:"price,omitempty" json:"price,omitempty"`
	ColorCards []*comm.MaterialConf `bson:"colorCards,omitempty" json:"colorCards,omitempty"`

	Categories []string `bson:"categories,omitempty" json:"categories,omitempty"`
	State      int32    `bson:"state,omitempty" json:"state,omitempty"`
}
